/*

    ConvertableData

    Class that allows access to elements of a DAL Data object 
    regardless of its type.

    (c) 1998 European Space Agency - written by Marco Beijersbergen
    
    Moved to separate utils package by H Siddiqui
	2001-01-02: UL: trival change to make optimizer happy

*/

#include "ConvertableData.h"
#include <sstream>

void ConvertableData::data(Data* cd)
{
  _type = cd->dataType();
  switch (_type) {
    case Dal::Int8: _data = cd->int8Data(); break;
    case Dal::Int16: _data = cd->int16Data(); break;
    case Dal::Uint16: _data = cd->uint16Data(); break;
    case Dal::Int32: _data = cd->int32Data(); break;
    case Dal::Uint32: _data = cd->uint32Data(); break;
    case Dal::Real32: _data = cd->real32Data(); break;
    case Dal::Real64: _data = cd->real64Data(); break;
    default:
      errstr << "cannot access data of type " << int(_type) << " as numeric data"
             << Sas::error("invalidDataType");
  }
}

double ConvertableData::asDouble(unsigned row)
{
  double r = 0.;

  assert(_data);

  switch (_type) {
    case Dal::Int8: r = (static_cast<int8*>(_data))[row]; break;
    case Dal::Int16: r = (static_cast<int16*>(_data))[row]; break;
    case Dal::Uint16: r = (static_cast<uint16*>(_data))[row]; break;
    case Dal::Int32: r = (static_cast<int32*>(_data))[row]; break;
    case Dal::Uint32: r = (static_cast<uint32*>(_data))[row]; break;
    case Dal::Real32: r = (static_cast<real32*>(_data))[row]; break;
    case Dal::Real64: r = (static_cast<real64*>(_data))[row]; break;
    default:
      errstr << "ConvertableData cannot return value of type " << int(_type)
             << " as a double" << Sas::error("invalidType");
  }
  return r;
}

string ConvertableData::operator[](unsigned row)
{
  stringstream ss;
  ss << asDouble(row);
  return ss.str();
}

bool isNumeric(Dal::DataType tp)
	{ return numericTypes.find(tp) != numericTypes.end(); }

