// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: GeneralIterator
// Author: Hassan Siddiqui
// 
// Modifications: See ChangeLog
// 
// $Id: GeneralIterator.h,v 1.1 2000/07/31 06:45:10 hsiddiqu Exp $

#ifndef GeneralIterator_H
#define GeneralIterator_H

// General classes

#include <iostream>

// SAS-specific classes


// General utility functions
#include "Tracer.h"

// ============================================================================

// class GeneralIterator

// An interface for traversing an aggregate of objects.

// ============================================================================

template <class Item>
class GeneralIterator
{
public:

  virtual void first()       = 0;
  virtual void next ()       = 0;
  virtual bool isDone()      = 0;
  virtual Item currentItem() = 0;

  virtual ~GeneralIterator(){};

protected:
  // Suppress default constructor
  GeneralIterator(){};

};

// ============================================================================

// FOREACH macro

// Define a simple macro simulating a foreach() method
// Note: Define with pointers as this is most often used

// ============================================================================

#define FOREACH(x) for ((*x).first(); !(*x).isDone(); (*x).next())
//#define FOREACH(x) for ( x.first(); ! x.isDone(); x.next())


// ============================================================================

// class RobustIterator

// A proxy class for GeneralIterator which provides garbage collection.
// RobustIterator is always placed on the STACK, which forces C++ 
// to automatically delete the class, and therefore the GeneralIterator object,
// when the instance RobustIterator in question goes out of scope.

// ============================================================================

template <class Item>
class RobustIterator
{
public:

  /* Constructor and destructor */
  RobustIterator(GeneralIterator<Item>* itr):
    _itr(itr)
    {};

  ~RobustIterator()
    {
      PROC;
//       cout << "deleting GeneralIterator object.." << endl;
      delete _itr;
      ENDPROC;
    }

  /* Accessors */

  GeneralIterator<Item> * operator->() { PROC; RETURN _itr;}
  GeneralIterator<Item> & operator*()  { PROC; RETURN (*_itr);}

protected:
  /* Suppress default constructor */
  RobustIterator(){};

private:
  /* Disallow copy and assignment fns
   * This prevents multiple deletions of same _itr
   */
  RobustIterator (const RobustIterator&);
  RobustIterator& operator=(const RobustIterator&);
  
  GeneralIterator<Item> * _itr;

};


#endif
