// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: ImagePixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of an iterator that handles DAL-based detectormaps.
//
// Modifications: See ChangeLog
// 
// $Id: ImagePixelIterator.cc,v 1.4 2001/03/20 17:12:04 hsiddiqu Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "ImagePixelIterator.h"

/* Constructors */
ImagePixelIterator::ImagePixelIterator(const string & arrayname) 
{
  /* Call the main iterator with invert_axes = false */
  ImagePixelIterator(arrayname, false);
}

ImagePixelIterator::ImagePixelIterator(const string & arrayname, 
bool invert_axes) :
  _firstcalled(false),
  _set(0),
  _array(0),
  _arraydata(0),
  // _arrayvalue(0),
  _wcsconv(0)
{

  _set = dataSetServer->
    open( arrayname, 
	  DataSet::Read );

  _array = _set->array(arrayname); 

  _size = _array->elements();
  
  /* Can only handle 2-d arrays */
  if (_array->size().size() !=2)
    {
      errstr << "ImagePixelIterator : cannot handle an array of size "
	     << _array->size().size()
	     << Sas::error ("invalidArraySize");

      ALTRETURN;
    }

  /* [HS] fixed bug in computation of array dimensions */
  _xbins=(_array->size())[1];

  errstr 
    << "xbins=" << _xbins
    << Sas::message(Msg::App, Msg::Noisy);


  _arraydata = _array->data();

  errstr 
    << "Array datatype " 
    << (long int ) _arraydata->dataType()
    << Sas::message(Msg::App, Msg::Verbose);


  _arrayvalue.data(_arraydata);
  
  /* Create WcsConvertor object */
  if (_array->attribute("CTYPE1")->asString().find("RA---TAN") != string::npos)
  {
     if (
      _array->hasAttribute("CRPIX1") &&
      _array->hasAttribute("CDELT1") &&
      _array->hasAttribute("CRPIX2") &&
      _array->hasAttribute("CDELT2") &&
      _set->hasAttribute("REFXCRPX") &&
      _set->hasAttribute("REFXCDLT") &&
      _set->hasAttribute("REFYCDLT") &&
      _set->hasAttribute("REFYCRPX") )
    {

    /* Set the widths and pixel converter from keywords - invert if necessary */
     double xwid=0.0, ywid=0.0;

     if (! invert_axes )
     {
       xwid = _array->attribute("CDELT1")->asReal() /
                         _set->attribute("REFXCDLT")->asReal();
       ywid = _array->attribute("CDELT2")->asReal() /
                         _set->attribute("REFYCDLT")->asReal();

       _wcsconv = new WCSConvertor("Sky", _array->attribute("CRPIX1")->asInt(),
                                        _set->attribute("REFXCRPX")->asReal(),
                                        xwid, 
                                 "Sky", _array->attribute("CRPIX2")->asInt(),
                                        _set->attribute("REFYCRPX")->asReal(),
                                        ywid);
     } 
     else {
  
       ywid = _array->attribute("CDELT1")->asReal() /
                         _set->attribute("REFXCDLT")->asReal();
       xwid = _array->attribute("CDELT2")->asReal() /
                         _set->attribute("REFYCDLT")->asReal();

       _wcsconv = new WCSConvertor("Sky", _array->attribute("CRPIX2")->asInt(),
                                        _set->attribute("REFYCRPX")->asReal(),
                                        xwid, 
                                 "Sky", _array->attribute("CRPIX1")->asInt(),
                                        _set->attribute("REFXCRPX")->asReal(),
                                        ywid);
     } 

    } else {
      
        errstr << "WCS info for the input file incomplete"
               << Sas::error("InvalidWCSinfo");
    }
  } else {
     _wcsconv = new WCSConvertor(_array);
  }

}

/* Destructor */
ImagePixelIterator::~ImagePixelIterator()
{
  PROC;
  
  delete _wcsconv;   _wcsconv=0;
  delete _arraydata; _arraydata=0;
 
  dataSetServer->close(_set); _set = 0;
 
  ENDPROC;
}

/* Iterator methods */
void ImagePixelIterator::first()
{
  _firstcalled = true;
  _index=0;
  _xindex=1;
  _yindex=1;
}


void ImagePixelIterator::next ()
{
  assert (!isDone());
  _index++;

  _xindex++;
  if (_xindex > _xbins)
    {
      _xindex=1;
      _yindex++;
    }
}

bool ImagePixelIterator::isDone()
{
  PROC;
  assert(_firstcalled);
  RETURN (_index >= _size);
}


PixelElement ImagePixelIterator::currentItem()
{
  PROC;

  assert(_firstcalled);

  /* Convert x and y positions to detector coordinates, 
     using WCS attributes */

  /* [HS] now usings new WCSConvertor interface */
  double value = _arrayvalue.asDouble(_index);
  CoordPair physcoord = _wcsconv->physicalCoords(_xindex, _yindex);
  PixelElement result (_xindex,
		       _yindex,
		       physcoord.x(),
		       physcoord.y(),
		       _wcsconv->xWidth(),
		       _wcsconv->yWidth(),
		      value);

  RETURN result;
}

unsigned ImagePixelIterator::size() const
{
  PROC;
  RETURN _size;
}

unsigned ImagePixelIterator::xbins() const
{
  PROC;
  RETURN _xbins;
}

/* WCSConvertor object */
const WCSConvertor * ImagePixelIterator::wcsconvertor() const
{
  PROC;
  RETURN _wcsconv;
}

/* Method to set the index position and start iterating from here */
void ImagePixelIterator::setIndex (unsigned long index)
{
    _index=index;
 
   _yindex=int(_index / _xbins) + 1;
   _xindex=_index + 1 - (_yindex - 1) * _xbins;
 
}
 
unsigned ImagePixelIterator::ybins() const
{
  PROC;
  RETURN _size / _xbins;
}


