// Module: WCSConvertor
//
// Author: Hassan Siddiqui
//
// Description: 
//
// A simple means of reading, converting and
//  propagating WCS information.
// Note that simple linear mapping is performed only.
//
// Modifications: See ChangeLog
// 
// $Id: WCSConvertor.h,v 1.6 2001/03/20 17:12:05 hsiddiqu Exp $

#ifndef WCSCONVERTOR_H
#define WCSCONVERTOR_H

/* General */

/* SAS-specific */
#include "Dal.h"
#include "errstr.h"
#include "Tracer.h"

/* Local classes */
#include "CoordPair.h"


class WCSConvertor
{
public:

  /*** Constructors */

  /* Read state from argument list */
  WCSConvertor( const string & ctype1,
		unsigned       crpix1,
		double         crval1,
		double         cdelt1,
		const string & ctype2,
		unsigned       crpix2,
		double         crval2,
		double         cdelt2 );


  /* Read attributes from an array */
  WCSConvertor(Array * block);

  /* Read attributes from table */
  WCSConvertor(const Column * xcol, const Column * ycol);

  /* Use 1:1 mapping for two input axis types */
  WCSConvertor(const string & xaxis, const string & yaxis);

  /* Default constructor */
  WCSConvertor();

  /* Destructors */
  ~WCSConvertor();

  /* Accessors */

  /* The physical coordinates corresponding to pixel position */
  CoordPair physicalCoords(double xpix, double ypix) const;
  CoordPair linearCoords(double xpix, double ypix) const;

  /* The pixel with in terms of the physical scale */
  double xWidth() const;
  double yWidth() const;

  /* The axes types */
  string xType() const;
  string yType() const;

  /* The physical coordinates at the pixel reference position */
  CoordPair physicalRefCoords() const;

  /* The pixel coordinates at the pixel reference position */
  CoordPair physicalRefPixel() const;

  /* Add attributes to an array */
  void addAttributesTo(Array * block) const;

  /* Add attributes to a table */
  void addAttributesTo(Column * xcol, Column * ycol) const;

private:
  bool check() const;
  
  string _ctype1;
  long   _crpix1;
  double _crval1;
  double _cdelt1;

  string _ctype2;
  long   _crpix2;
  double _crval2;
  double _cdelt2;
};

#endif
